#
# est-multiplier.R, 19 Jun 22
#
# Data from:
# A conversation around the analysis of the {SiP} effort estimation dataset
# Derek M. Jones and Stephen Cullum

library("colorspace")
library("plyr")


par(bty="l")
par(las=1)
par(pch="+")
pal_col=rainbow(4)

# Calculate a weighted mean, and low/high bounds on over/under
ov_un_eq=function(sd=1)
{
# Calculate prediction confidence interval
MSE=sum(Sip_mod$residuals^2)/(length(Sip_mod$residuals)-2)
# Variances, but not sd, can be added
pred_se=sqrt(pred$se.fit^2+MSE)

# Low/high estimate bounds
x_bounds=c(0.5, 500)
pred_bnds=predict(Sip_mod, newdata=data.frame(HoursEstimate=x_bounds), se.fit=TRUE)
pred_bnds_se=sqrt(pred_bnds$se.fit^2+MSE)

# Weight over/under estimation factor by number of estimates
# having a given value
# Predict all estimates
pred=predict(Sip_mod, se.fit=TRUE)
pred_se=sqrt(pred$se.fit^2+MSE)

x_un=exp(pred_bnds$fit+sd*pred_bnds_se)/x_bounds
x_ov=exp(pred_bnds$fit-sd*pred_bnds_se)/x_bounds

# 'sd' standard deviation
e_un=exp(pred$fit+sd*pred_se)/Sip$HoursEstimate
e_ov=exp(pred$fit-sd*pred_se)/Sip$HoursEstimate

print("Over")
print(mean(e_ov))
print(x_ov)
print("Under")
print(mean(e_un))
print(x_un)

return(NULL)
}


Sip_all=read.csv("Sip-task-info.csv.xz", as.is=TRUE)

Sip_uTN=subset(Sip, !duplicated(TaskNumber)) # Keep one instance of each multi-person task
# Removed the 190 tasks that were cancelled before completion
Sip=subset(Sip_uTN, StatusCode != "CANCELLED")
# Single instance of this in the data
Sip=subset(Sip, StatusCode != "TEMPLATE")
# Projects that were not completed at the time of the data snapshot
Sip=subset(Sip, StatusCode != "CHRONICLE")

plot(Sip$HoursEstimate, Sip$HoursActual, log="xy", col=pal_col[3],
	xlim=c(0.2, 1e3), ylim=c(0.2, 1e3),
	xlab="Estimate (hours)", ylab="Actual (hours)\n")
lines(c(1e-2, 1e3), c(1e-2, 1e3), col="grey")

Sip_mod=glm(log(HoursActual) ~ log(HoursEstimate), data=Sip)
summary(Sip_mod)

x_range=exp(seq(log(0.2), log(1e3), by=0.5))
pred=predict(Sip_mod, newdata=data.frame(HoursEstimate=x_range), se.fit=TRUE)

lines(x_range, exp(pred$fit), col=pal_col[1])

# Calculate prediction confidence interval
MSE=sum(Sip_mod$residuals^2)/(length(Sip_mod$residuals)-2)
# Variances, but not sd, can be added
pred_se=sqrt(pred$se.fit^2+MSE)

# One standard deviation, 68.5% confidence interval
x_un=exp(pred$fit+pred_se)/x_range
x_ov=exp(pred$fit-pred_se)/x_range

# One standard deviation, 68.5% confidence interval
lines(x_range, exp(pred$fit+1*pred_se), col=pal_col[4])
lines(x_range, exp(pred$fit-1*pred_se), col=pal_col[4])

# 95% confidence interval, almost two standard deviations
lines(x_range, exp(pred$fit+1.96*pred_se), col=pal_col[2])
lines(x_range, exp(pred$fit-1.96*pred_se), col=pal_col[2])

legend(x="topleft", legend=c("Fitted model", "95% prediction interval", "SiP tasks", "1 sd (68%) prediction interval"),
			bty="n", fill=pal_col, cex=1.3)

ov_un_eq(1)  # 1 standard deviation
ov_un_eq(1.96) # 95%, close to 2 standard deviations

# Separate out over/under and exact estimates
over=subset(Sip, HoursActual < HoursEstimate)
under=subset(Sip, HoursEstimate < HoursActual)
exact=subset(Sip, HoursEstimate == HoursActual)

# Geometric means
exp(sum(log(under$HoursActual/under$HoursEstimate))/nrow(under))
# [1] 1.920468
exp(sum(log(over$HoursActual/over$HoursEstimate))/nrow(over))
# [1] 0.4677265

100*c(nrow(under), nrow(exact), nrow(over))/nrow(Sip)

pal_col=rainbow(2)

plot(1, type="n", log="xy",
	yaxs="i",
	xlim=c(0.2, 1e3), ylim=c(0.1, 10),
	xlab="Estimate (hours)", ylab="Actual/Estimate\n")

points(over$HoursEstimate, over$HoursActual/over$HoursEstimate, col=pal_col[1])
points(under$HoursEstimate, under$HoursActual/under$HoursEstimate, col=pal_col[2])

# Quantile regression


library("quantreg")


plot(Sip$HoursEstimate, Sip$HoursActual, log="xy", col=pal_col[2],
	xlim=c(0.2, 1e3), ylim=c(0.2, 1e3),
	xlab="Estimate (hours)", ylab="Actual (hours)\n")
lines(c(1e-2, 1e3), c(1e-2, 1e3), col=pal_col[3])

# Fit at upper bound
Sip_hqmod=rq(log(HoursActual) ~ log(HoursEstimate), data=Sip, tau=0.975)
summary(Sip_hqmod)
pred_hq=predict(Sip_hqmod, newdata=data.frame(HoursEstimate=x_range))
lines(x_range, exp(pred_hq), col=pal_col[1])

# Fit at lower bound
Sip_lqmod=rq(log(HoursActual) ~ log(HoursEstimate), data=Sip, tau=0.025)
summary(Sip_lqmod)
pred_lq=predict(Sip_lqmod, newdata=data.frame(HoursEstimate=x_range))
lines(x_range, exp(pred_lq), col=pal_col[1])


